/**
 * vue-nav-tab v2.5.0
 *     @author Phạm Trung Kiên
 *     @link https://github.com/kien5436/vue-nav-tab.git
 *     @license MIT
 */

'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');

class VueNavTabError extends Error {
    constructor(message) {
        super(message);
        Object.setPrototypeOf(this, VueNavTabError.prototype);
        this.name = "VueNavTabError";
    }
}
const MISSING_REQUIRED_PROPS = "Some properties of the tab are missing";

var script$8 = vue.defineComponent({ name: "BlankTabView" });

const _hoisted_1$7 = { class: "vp-box-border" };

function render$8(_ctx, _cache, $props, $setup, $data, $options) {
  return (vue.openBlock(), vue.createElementBlock("div", _hoisted_1$7))
}

script$8.render = render$8;

const tabsGroup = vue.reactive({});
const removedTabsGroup = {};
const BlankTab = {
    active: true,
    id: "_",
    title: "Blank",
    view: vue.markRaw(script$8),
};
function createTabs(group, tabs) {
    if (!Array.isArray(tabs)) {
        throw new VueNavTabError("'tabs' must be an array");
    }
    const subGroups = group.split(".");
    if (1 < subGroups.length) {
        for (let i = 0, len = subGroups.length - 1; i < len; i++) {
            if (!tabsGroup.hasOwnProperty(subGroups[i])) {
                throw new VueNavTabError(`'${subGroups[i]}' group is missing while creating nested tabs. Do you forget to create it?`);
            }
        }
    }
    if (tabsGroup.hasOwnProperty(group)) {
        throw new VueNavTabError(`'${group}' group was created. If you want to add more tabs into it, use 'addTab' instead`);
    }
    if (tabs.some((_tab) => !isValidTab(_tab))) {
        throw new VueNavTabError(MISSING_REQUIRED_PROPS);
    }
    const _tabs = vue.ref(tabs.map((tab) => (Object.assign(Object.assign({}, tab), { _key: tab.id + "~", active: !!tab.active, closable: !!tab.closable, hasIframe: !!tab.hasIframe, title: !isPrimitiveType(tab.title) ? vue.markRaw(tab.title) : tab.title.toString(), view: vue.markRaw(tab.view) }))));
    if (0 === _tabs.value.length) {
        _tabs.value.push(BlankTab);
    }
    tabsGroup[group] = _tabs;
    removedTabsGroup[group] = [];
    return tabsGroup[group];
}
function useTabs(group) {
    const tabs = tabsGroup[group];
    if (undefined === tabs) {
        throw new VueNavTabError(`Group '${group}' is undefined`);
    }
    return tabs;
}
function useCurrentTab(group) {
    const tabs = useTabs(group);
    return vue.computed(() => tabs.find((tab) => tab.active));
}
/**
 * Add a tab to a group or replace existed one
 * @param group group that tab is added
 * @param tab the tab to be added
 */
function addTab(group, tab) {
    if (!isValidTab(tab)) {
        throw new VueNavTabError(MISSING_REQUIRED_PROPS);
    }
    const tabs = useTabs(group);
    const cookedTab = Object.assign(Object.assign({}, tab), { _key: tab.id + "~", active: !!tab.active, closable: !!tab.closable, hasIframe: !!tab.hasIframe, title: !isPrimitiveType(tab.title) ? vue.markRaw(tab.title) : tab.title.toString(), view: vue.markRaw(tab.view) });
    let wasRemoved = false;
    for (let i = removedTabsGroup[group].length; 0 <= --i;) {
        if (removedTabsGroup[group][i] === tab.id) {
            wasRemoved = true;
            cookedTab._key = `${cookedTab.id}~${Date.now()}`;
            removedTabsGroup[group].splice(i, 1);
            break;
        }
    }
    if (1 === tabs.length && BlankTab.id === tabs[0].id) {
        cookedTab.active = true;
        tabs.splice(0, 1, cookedTab);
        return;
    }
    const currentTab = useCurrentTab(group);
    const idx = tabs.findIndex((_tab) => _tab.id === tab.id);
    const currentTabIdx = tabs.findIndex((_tab) => _tab.id === currentTab.value.id);
    if (cookedTab.active && idx !== currentTabIdx) {
        tabs.splice(currentTabIdx, 1, Object.assign(Object.assign({}, currentTab.value), { active: false }));
    }
    if (!cookedTab.active && idx === currentTabIdx) {
        cookedTab.active = true;
    }
    if (0 > idx || wasRemoved) {
        tabs.push(cookedTab);
    }
    else {
        cookedTab._key = tabs[idx]._key;
        tabs.splice(idx, 1, cookedTab);
    }
}
function removeTab(group, tabId) {
    const tabs = useTabs(group);
    const currentTab = useCurrentTab(group);
    const idx = tabs.findIndex((tab) => tab.id === tabId);
    const currentTabIdx = tabs.findIndex((tab) => tab.id === currentTab.value.id);
    if (0 > idx || !tabs[idx].closable) {
        return;
    }
    removedTabsGroup[group].push(tabId);
    if (1 === tabs.length) {
        tabs.push(BlankTab);
    }
    if (idx !== currentTabIdx) {
        tabs.splice(idx, 1);
        return;
    }
    if (currentTab.value.active) {
        const prevTabIdx = currentTabIdx - 1;
        const nextTabIdx = currentTabIdx + 1;
        if (0 > prevTabIdx) {
            tabs.splice(nextTabIdx, 1, Object.assign(Object.assign({}, tabs[nextTabIdx]), { active: true }));
        }
        else {
            tabs.splice(prevTabIdx, 1, Object.assign(Object.assign({}, tabs[prevTabIdx]), { active: true }));
        }
    }
    tabs.splice(currentTabIdx, 1);
}
function refreshTab(group, tabId) {
    const tabs = useTabs(group);
    const selectedTabIdx = tabs.findIndex((_tab) => _tab.id === tabId);
    if (0 > selectedTabIdx) {
        throw new VueNavTabError(`'${tabId}' doesn't exist in group '${group}'`);
    }
    tabs.splice(selectedTabIdx, 1, Object.assign(Object.assign({}, tabs[selectedTabIdx]), { _key: `${tabId}~${Date.now()}` }));
    for (const key in tabsGroup) {
        if (tabsGroup.hasOwnProperty(key) && key.startsWith(group) && key !== group) {
            delete tabsGroup[key];
        }
    }
}
function isPrimitiveType(value) {
    return (null === value ||
        undefined === value ||
        "bigint" === typeof value ||
        "boolean" === typeof value ||
        "number" === typeof value ||
        "string" === typeof value ||
        "symbol" === typeof value);
}
function isValidTab(tab) {
    return tab.hasOwnProperty("id") && tab.hasOwnProperty("title") && tab.hasOwnProperty("view");
}
function removeAll(group) {
    const tabs = useTabs(group);
    let currentTabIsRemoved = false;
    for (let i = 0, len = tabs.length; i < len; i++) {
        const tab = tabs[i];
        if (tab.closable) {
            if (!currentTabIsRemoved) {
                currentTabIsRemoved = !!tab.active;
            }
            removedTabsGroup[group].push(tab.id);
            tabs.splice(i, 1);
            i--, len--;
        }
    }
    if (0 === tabs.length) {
        tabs.push(BlankTab);
    }
    else if (currentTabIsRemoved) {
        tabs.splice(0, 1, Object.assign(Object.assign({}, tabs[0]), { active: true }));
    }
}
function removeLeft(group, tabId) {
    const tabs = useTabs(group);
    let idx = tabs.findIndex((tab) => tab.id === tabId);
    if (0 > idx) {
        return;
    }
    let currentTabIsRemoved = false;
    for (let i = 0; i < idx; i++) {
        const tab = tabs[i];
        if (tab.closable) {
            if (!currentTabIsRemoved) {
                currentTabIsRemoved = !!tab.active;
            }
            removedTabsGroup[group].push(tab.id);
            tabs.splice(i, 1);
            i--, idx--;
        }
    }
    if (currentTabIsRemoved) {
        tabs.splice(idx, 1, Object.assign(Object.assign({}, tabs[idx]), { active: true }));
    }
}
function removeRight(group, tabId) {
    const tabs = useTabs(group);
    const idx = tabs.findIndex((tab) => tab.id === tabId);
    if (0 > idx) {
        return;
    }
    let currentTabIsRemoved = false;
    for (let i = idx + 1, len = tabs.length; i < len; i++) {
        const tab = tabs[i];
        if (tab.closable) {
            if (!currentTabIsRemoved) {
                currentTabIsRemoved = !!tab.active;
            }
            removedTabsGroup[group].push(tab.id);
            tabs.splice(i, 1);
            i--, len--;
        }
    }
    if (currentTabIsRemoved) {
        tabs.splice(idx, 1, Object.assign(Object.assign({}, tabs[idx]), { active: true }));
    }
}
function removeOthers(group, tabId) {
    removeLeft(group, tabId);
    removeRight(group, tabId);
}
function removeAbove(group, tabId) {
    removeLeft(group, tabId);
}
function removeBelow(group, tabId) {
    removeRight(group, tabId);
}
// eslint-disable-next-line no-unused-vars
function onTabActivated(group, callback) {
    const currentTab = vue.toRaw(useCurrentTab(group).value);
    vue.onActivated(() => callback(currentTab));
}
// eslint-disable-next-line no-unused-vars
function onTabDeactivated(group, callback) {
    const currentTab = vue.toRaw(useCurrentTab(group).value);
    vue.onDeactivated(() => callback(currentTab));
}

const defaultLocales = {
    en: {
        close: "Close",
        closeAbove: "Close to the above",
        closeAll: "Close all",
        closeBelow: "Close to the below",
        closeLeft: "Close to the left",
        closeOthers: "Close others",
        closeRight: "Close to the right",
        reload: "Reload",
    },
    vi: {
        close: "Đóng",
        closeAbove: "Đóng các thẻ bên trên",
        closeAll: "Đóng tất cả",
        closeBelow: "Đóng các thẻ bên dưới",
        closeLeft: "Đóng các thẻ bên trái",
        closeOthers: "Đóng các thẻ khác",
        closeRight: "Đóng các thẻ bên phải",
        reload: "Tải lại",
    },
};
const key = "vnt-i18n";
function mergeWithDefaultLocales(options) {
    const i18n = Object.assign({}, defaultLocales);
    const requiredProps = Object.keys(defaultLocales.en);
    for (const lang in options) {
        if (options.hasOwnProperty(lang)) {
            const messages = options[lang];
            for (const prop of requiredProps) {
                if (!messages.hasOwnProperty(prop) || "string" !== typeof (messages[prop])) {
                    throw new VueNavTabError(`'${prop}' is missing from 'options.${lang}' or has mismatch type. Possible values are {
            close: string;
            closeAbove: string;
            closeAll: string;
            closeBelow: string;
            closeLeft: string;
            closeOthers: string;
            closeRight: string;
            reload: string;
          }`);
                }
            }
            i18n[lang] = messages;
        }
    }
    return i18n;
}
function useContextMenu(group, navType) {
    const contextMenu = vue.ref(null);
    let tabContext = null;
    const i18n = vue.inject(key, defaultLocales);
    const lang = vue.ref("en");
    const disabledActions = vue.reactive({
        close: false,
        closeAbove: false,
        closeBelow: false,
        closeLeft: false,
        closeRight: false,
    });
    const actions = vue.computed(() => [
        {
            disabled: false,
            id: "reload",
            text: i18n[lang.value].reload,
        },
        {
            disabled: disabledActions.close,
            id: "close",
            text: i18n[lang.value].close,
        },
        {
            disabled: false,
            id: "closeAll",
            text: i18n[lang.value].closeAll,
        },
        {
            disabled: false,
            id: "closeOthers",
            text: i18n[lang.value].closeOthers,
        },
        {
            disabled: disabledActions.closeAbove || disabledActions.closeLeft,
            id: "vertical" === navType ? "closeAbove" : "closeLeft",
            text: "vertical" === navType ? i18n[lang.value].closeAbove : i18n[lang.value].closeLeft,
        },
        {
            disabled: disabledActions.closeBelow || disabledActions.closeRight,
            id: "vertical" === navType ? "closeBelow" : "closeRight",
            text: "vertical" === navType ? i18n[lang.value].closeBelow : i18n[lang.value].closeRight,
        },
    ]);
    function showContextMenu(e, tabId) {
        var _a;
        const documentLang = document.documentElement.lang;
        lang.value = i18n.hasOwnProperty(documentLang) ? documentLang : "en";
        (_a = contextMenu.value) === null || _a === void 0 ? void 0 : _a.open(e);
        tabContext = tabId;
        disableActions(tabId);
    }
    function onMenuClick(actionId) {
        var _a;
        (_a = contextMenu.value) === null || _a === void 0 ? void 0 : _a.close();
        if (!tabContext) {
            return;
        }
        switch (actionId) {
            case "reload":
                refreshTab(group, tabContext);
                break;
            case "close":
                removeTab(group, tabContext);
                break;
            case "closeAll":
                removeAll(group);
                break;
            case "closeOthers":
                removeOthers(group, tabContext);
                break;
            case "closeLeft":
            case "closeAbove":
                removeLeft(group, tabContext);
                break;
            case "closeRight":
            case "closeBelow":
                removeRight(group, tabContext);
                break;
        }
    }
    function disableActions(tabId) {
        const tabs = useTabs(group);
        const selectedTabIdx = tabs.findIndex((tab) => tab.id === tabId);
        const selectedTab = tabs[selectedTabIdx];
        disabledActions.closeLeft = disabledActions.closeAbove = 0 === selectedTabIdx;
        disabledActions.closeRight = disabledActions.closeBelow = tabs.length - 1 === selectedTabIdx;
        disabledActions.close = !selectedTab.closable;
    }
    return {
        actions,
        contextMenu,
        onMenuClick,
        showContextMenu,
    };
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

// eslint-disable-next-line no-unused-vars
function debounce(func, wait = 250, immediate = false) {
    let timeout = null;
    function debounced(...args) {
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        let context = this;
        const previous = Date.now();
        function later() {
            const passed = Date.now() - previous;
            if (wait > passed) {
                timeout = setTimeout(later, wait - passed);
            }
            else {
                timeout = null;
                if (!immediate)
                    func.apply(context, args);
                // this check is needed because `func` can recursively invoke debounce
                if (!timeout)
                    context = null;
            }
        }
        if (!timeout) {
            timeout = setTimeout(later, wait);
            if (immediate)
                func.apply(context, args);
        }
    }
    debounced.cancel = function () {
        if (!timeout)
            return;
        clearTimeout(timeout);
        timeout = null;
    };
    return debounced;
}

var script$7 = vue.defineComponent({
    name: "ContextMenu",
    props: {
        boundingView: {
            default: null,
            type: String,
        },
    },
    setup(props) {
        const isOpen = vue.ref(false);
        const el = vue.ref(null);
        const coord = vue.reactive({
            left: 0,
            top: 0,
        });
        const boundingEl = vue.computed(() => {
            var _a;
            return (props.boundingView
                ? (_a = el.value) === null || _a === void 0 ? void 0 : _a.closest(props.boundingView)
                : document.documentElement);
        });
        const style = vue.computed(() => ({
            display: isOpen.value ? "block" : "none",
            left: coord.left + "px",
            top: coord.top + "px",
            zIndex: 10000,
        }));
        const debouncedClose = debounce(close, 100, true);
        vue.onMounted(() => {
            document.addEventListener("keyup", onKeyUp, false);
            document.addEventListener("click", close, false);
            document.addEventListener("scroll", debouncedClose, false);
            boundingEl.value.addEventListener("scroll", debouncedClose, false);
            window.addEventListener("resize", debouncedClose, false);
        });
        vue.onUnmounted(() => {
            document.removeEventListener("keyup", onKeyUp, false);
            document.removeEventListener("click", close, false);
            document.removeEventListener("scroll", debouncedClose, false);
            boundingEl.value.removeEventListener("scroll", debouncedClose, false);
            window.removeEventListener("resize", debouncedClose, false);
        });
        function open(e) {
            if (!el.value)
                return;
            el.value.style.setProperty("display", "");
            const { height, width } = el.value.getBoundingClientRect();
            coord.left = width + e.clientX + 20 > document.documentElement.clientWidth ? e.clientX - width : e.clientX;
            coord.top = height + e.clientY + 20 > document.documentElement.clientHeight ? e.clientY - height : e.clientY;
            vue.nextTick(() => { var _a; return (_a = el.value) === null || _a === void 0 ? void 0 : _a.focus(); });
            isOpen.value = true;
        }
        function close() {
            isOpen.value = false;
        }
        function onKeyUp(e) {
            if ("Escape" === e.code)
                close();
        }
        return {
            close,
            el,
            open,
            style,
        };
    },
});

function render$7(_ctx, _cache, $props, $setup, $data, $options) {
  return (vue.openBlock(), vue.createElementBlock("div", {
    ref: "el",
    class: "vp-fixed vp-outline-none vp-font-sans vp-select-none",
    tabindex: "-1",
    style: vue.normalizeStyle(_ctx.style)
  }, [
    vue.renderSlot(_ctx.$slots, "default")
  ], 4))
}

script$7.render = render$7;

var script$6 = vue.defineComponent({
    name: "ContextMenuList",
    props: {
        actions: {
            required: true,
            type: Array,
        },
        onMenuClick: {
            required: true,
            type: Function,
        },
    },
});

const _hoisted_1$6 = {
  class: "vp-bg-white vp-list-none vp-m-0 vp-px-0 vp-py-2 vp-shadow-md vp-rounded vp-text-left",
  role: "menu"
};
const _hoisted_2$1 = ["onClick"];

function render$6(_ctx, _cache, $props, $setup, $data, $options) {
  return (vue.openBlock(), vue.createElementBlock("ul", _hoisted_1$6, [
    (vue.openBlock(true), vue.createElementBlock(vue.Fragment, null, vue.renderList(_ctx.actions, (action) => {
      return (vue.openBlock(), vue.createElementBlock("li", {
        class: vue.normalizeClass(["vp-py-[.375rem] vp-px-4 vp-whitespace-nowrap vp-text-gray-600 vp-text-sm vp-transition-colors", {
      'vp-text-gray-400': action.disabled,
      'hover:vp-bg-gray-100 hover:vp-text-gray-800': !action.disabled,
    }]),
        key: action.id,
        onClick: $event => (_ctx.onMenuClick(action.id))
      }, vue.toDisplayString(action.text), 11, _hoisted_2$1))
    }), 128))
  ]))
}

script$6.render = render$6;

var script$5 = vue.defineComponent({
    name: "Tab",
    props: {
        active: {
            default: false,
            type: Boolean,
        },
        closable: {
            default: false,
            type: Boolean,
        },
        direction: {
            required: true,
            type: String,
            validator: (val) => ["top", "bottom", "none"].includes(val),
        },
        group: {
            required: true,
            type: String,
        },
        rounded: {
            default: false,
            type: Boolean,
        },
        tabId: {
            required: true,
            type: String,
        },
    },
    setup(props) {
        const tabs = useTabs(props.group);
        const currentTab = useCurrentTab(props.group);
        const classes = vue.computed(() => ({
            "vp-item-active": props.active,
            "vp-pr-4 vp-justify-center": !props.closable,
            "vp-pr-8 vp-justify-start": props.closable,
            "vp-rounded": props.rounded && "none" === props.direction,
            "vp-rounded-b": props.rounded && "top" === props.direction,
            "vp-rounded-t": props.rounded && "bottom" === props.direction,
            "vp-text-gray-100 !vp-text-gray-500 !vp-bg-gray-100 vp-pointer-events-none": "_" === props.tabId,
        }));
        const computedTabId = vue.computed(() => `${props.tabId}-tab`);
        const tab = vue.ref(null);
        vue.onMounted(() => { var _a; return props.active && ((_a = tab.value) === null || _a === void 0 ? void 0 : _a.scrollIntoView()); });
        vue.onUpdated(() => { var _a; return props.active && !isVisible() && ((_a = tab.value) === null || _a === void 0 ? void 0 : _a.scrollIntoView()); });
        function activateTab() {
            if (props.active)
                return;
            const idx = tabs.findIndex((_tab) => _tab.id === props.tabId);
            const currentTabIdx = tabs.findIndex((_tab) => _tab.id === currentTab.value.id);
            tabs.splice(currentTabIdx, 1, Object.assign(Object.assign({}, currentTab.value), { active: false }));
            tabs.splice(idx, 1, Object.assign(Object.assign({}, tabs[idx]), { active: true }));
        }
        function closeTab() {
            removeTab(props.group, props.tabId);
        }
        function isVisible() {
            var _a, _b, _c, _d, _e;
            const { top, left, bottom, right } = (_a = tab.value) === null || _a === void 0 ? void 0 : _a.getBoundingClientRect();
            if ("none" === props.direction) {
                const { top: viewTop, bottom: viewBottom } = (_c = (_b = tab.value) === null || _b === void 0 ? void 0 : _b.closest(".vp-nav")) === null || _c === void 0 ? void 0 : _c.getBoundingClientRect();
                return !(bottom < viewTop || top > viewBottom);
            }
            else {
                const { left: viewLeft, right: viewRight } = (_e = (_d = tab.value) === null || _d === void 0 ? void 0 : _d.closest(".vp-nav")) === null || _e === void 0 ? void 0 : _e.getBoundingClientRect();
                return !(left > viewRight || right < viewLeft);
            }
        }
        return {
            activateTab,
            classes,
            closeTab,
            computedTabId,
            tab,
        };
    },
});

const _hoisted_1$5 = ["id"];
const _hoisted_2 = { class: "vp-truncate vp-text-clip vp-flex vp-items-center vp-box-border" };
const _hoisted_3 = /*#__PURE__*/vue.createElementVNode("i", { class: "vp-icon vp-icon-x" }, null, -1);
const _hoisted_4 = [
  _hoisted_3
];

function render$5(_ctx, _cache, $props, $setup, $data, $options) {
  return (vue.openBlock(), vue.createElementBlock("a", {
    ref: "tab",
    class: vue.normalizeClass(["vp-flex vp-text-gray-600 vp-items-center !vp-no-underline vp-h-10 vp-box-border vp-transition vp-pl-4 vp-absolute vp-cursor-pointer vp-font-sans vp-item", _ctx.classes]),
    id: _ctx.computedTabId,
    draggable: "true",
    onClick: _cache[1] || (_cache[1] = (...args) => (_ctx.activateTab && _ctx.activateTab(...args)))
  }, [
    vue.createElementVNode("div", _hoisted_2, [
      vue.renderSlot(_ctx.$slots, "default")
    ]),
    (_ctx.closable)
      ? (vue.openBlock(), vue.createElementBlock("span", {
          key: 0,
          class: "vp-absolute vp-top-2 vp-right-1 vp-w-6 vp-h-6 hover:vp-bg-gray-200 vp-rounded-sm vp-inline-flex vp-items-center vp-justify-center vp-box-border",
          onClick: _cache[0] || (_cache[0] = vue.withModifiers((...args) => (_ctx.closeTab && _ctx.closeTab(...args)), ["stop","prevent"]))
        }, _hoisted_4))
      : vue.createCommentVNode("", true)
  ], 10, _hoisted_1$5))
}

script$5.render = render$5;

var script$4 = vue.defineComponent({
    name: "XScrollbar",
    setup() {
        const navWrapper = vue.ref(null);
        const track = vue.ref(null);
        const thumb = vue.ref(null);
        const scrollRatio = vue.ref(1);
        const showScrollbar = vue.ref(false);
        const thumbStyle = vue.computed(() => (!showScrollbar.value || 1 === scrollRatio.value ? null : { width: `${scrollRatio.value * 100}%` }));
        const navWrapperScrollPos = vue.reactive({
            left: 0,
            x: 0,
        });
        function startGrabThumb(e) {
            navWrapperScrollPos.left = navWrapper.value ? navWrapper.value.scrollLeft : 0;
            navWrapperScrollPos.x = e.clientX;
            document.body.style.setProperty("user-select", "none");
            document.addEventListener("mousemove", moveThumb, false);
            document.addEventListener("mouseup", stopGrabThumb, false);
        }
        function moveThumb(e) {
            if (navWrapper.value) {
                navWrapper.value.scrollLeft = (e.clientX - navWrapperScrollPos.x) / scrollRatio.value + navWrapperScrollPos.left;
            }
        }
        function stopGrabThumb() {
            document.body.style.removeProperty("user-select");
            document.removeEventListener("mousemove", moveThumb, false);
            document.removeEventListener("mouseup", stopGrabThumb, false);
        }
        function scrollThumb() {
            window.requestAnimationFrame(() => {
                if (navWrapper.value && thumb.value) {
                    thumb.value.style.setProperty("left", `${navWrapper.value.scrollLeft / navWrapper.value.scrollWidth * 100}%`);
                }
            });
        }
        function onClickTrack(e) {
            if (!track.value || !navWrapper.value)
                return;
            const trackRect = track.value.getBoundingClientRect();
            const percentage = (e.clientX - trackRect.left) / trackRect.width;
            navWrapper.value.scrollLeft = percentage * (navWrapper.value.scrollWidth - navWrapper.value.clientWidth);
        }
        function trackUserMovement(e) {
            var _a;
            if (!navWrapper.value)
                return;
            scrollRatio.value = navWrapper.value.clientWidth / navWrapper.value.scrollWidth;
            showScrollbar.value = "mouseenter" === e.type && 1 > scrollRatio.value;
            (_a = thumb.value) === null || _a === void 0 ? void 0 : _a.style.setProperty("left", `${navWrapper.value.scrollLeft / navWrapper.value.scrollWidth * 100}%`);
        }
        return {
            navWrapper,
            onClickTrack,
            scrollThumb,
            showScrollbar,
            startGrabThumb,
            thumb,
            thumbStyle,
            track,
            trackUserMovement,
        };
    },
});

const _hoisted_1$4 = { class: "vp-absolute vp-h-1.5 vp-left-0 vp-w-full vp-bottom-0" };

function render$4(_ctx, _cache, $props, $setup, $data, $options) {
  return (vue.openBlock(), vue.createElementBlock("div", {
    class: "vp-overflow-hidden vp-h-10 vp-relative",
    onMouseenter: _cache[3] || (_cache[3] = (...args) => (_ctx.trackUserMovement && _ctx.trackUserMovement(...args))),
    onMouseleave: _cache[4] || (_cache[4] = (...args) => (_ctx.trackUserMovement && _ctx.trackUserMovement(...args)))
  }, [
    vue.createElementVNode("div", {
      ref: "navWrapper",
      class: "vp-h-full vp-overflow-x-auto vp-overflow-y-hidden -vp-mb-4 vp-pb-4 vp-box-content",
      onScroll: _cache[0] || (_cache[0] = (...args) => (_ctx.scrollThumb && _ctx.scrollThumb(...args)))
    }, [
      vue.renderSlot(_ctx.$slots, "default")
    ], 544),
    vue.withDirectives(vue.createElementVNode("div", _hoisted_1$4, [
      vue.createElementVNode("div", {
        ref: "track",
        class: "vp-absolute vp-top-0 vp-left-0 vp-w-full vp-h-full vp-bg-slate-400/30 vp-rounded",
        onClick: _cache[1] || (_cache[1] = (...args) => (_ctx.onClickTrack && _ctx.onClickTrack(...args)))
      }, null, 512),
      vue.createElementVNode("div", {
        ref: "thumb",
        class: "vp-absolute vp-top-0 vp-h-full vp-bg-zinc-400 vp-rounded",
        style: vue.normalizeStyle(_ctx.thumbStyle),
        onMousedown: _cache[2] || (_cache[2] = (...args) => (_ctx.startGrabThumb && _ctx.startGrabThumb(...args)))
      }, null, 36)
    ], 512), [
      [vue.vShow, _ctx.showScrollbar]
    ])
  ], 32))
}

script$4.render = render$4;

function useDragDrop(group) {
    const tabs = useTabs(group);
    function dragStart(e) {
        var _a, _b;
        let id = e.target.id;
        id = id.slice(0, id.lastIndexOf("-tab"));
        (_a = e.dataTransfer) === null || _a === void 0 ? void 0 : _a.setData("text/vnt-group", group);
        (_b = e.dataTransfer) === null || _b === void 0 ? void 0 : _b.setData("text/plain", id);
    }
    function dragOver(e) {
        var _a, _b;
        if (group !== ((_a = e.dataTransfer) === null || _a === void 0 ? void 0 : _a.getData("text/vnt-group")))
            return;
        e.preventDefault();
        const draggedItemId = (_b = e.dataTransfer) === null || _b === void 0 ? void 0 : _b.getData("text/plain");
        const draggedIdx = tabs.findIndex((tab) => tab.id === draggedItemId);
        const targetItem = e.target.closest(".vp-item");
        if (null === targetItem)
            return;
        const targetId = targetItem.id.slice(0, targetItem.id.lastIndexOf("-tab"));
        const targetIdx = tabs.findIndex((tab) => tab.id === targetId);
        if (draggedIdx !== targetIdx) {
            [tabs[draggedIdx], tabs[targetIdx]] = [tabs[targetIdx], tabs[draggedIdx]];
            tabs.splice(0, 0);
        }
    }
    function drop(e) {
        var _a, _b;
        e.preventDefault();
        if (group !== ((_a = e.dataTransfer) === null || _a === void 0 ? void 0 : _a.getData("text/vnt-group")))
            return;
        const draggedItemId = (_b = e.dataTransfer) === null || _b === void 0 ? void 0 : _b.getData("text/plain");
        const draggedIdx = tabs.findIndex((tab) => tab.id === draggedItemId);
        const targetItem = e.target.closest(".vp-item");
        if (null === targetItem)
            return;
        const targetId = targetItem.id.slice(0, targetItem.id.lastIndexOf("-tab"));
        const targetIdx = tabs.findIndex((tab) => tab.id === targetId);
        if (draggedIdx !== targetIdx) {
            [tabs[draggedIdx], tabs[targetIdx]] = [tabs[targetIdx], tabs[draggedIdx]];
            tabs.splice(0, 0);
        }
    }
    return {
        dragOver,
        dragStart,
        drop,
    };
}

var script$3 = vue.defineComponent({
    name: "HTabs",
    components: {
        ContextMenu: script$7,
        ContextMenuList: script$6,
        Tab: script$5,
        XScrollbar: script$4,
    },
    props: {
        appearance: {
            default: "tabs",
            type: String,
            validator: (val) => ["tabs", "pills"].includes(val),
        },
        group: {
            required: true,
            type: String,
        },
        position: {
            default: "bottom",
            type: String,
            validator: (val) => ["top", "bottom"].includes(val),
        },
        rounded: {
            default: false,
            type: Boolean,
        },
        tabs: {
            required: true,
            type: Array,
        },
    },
    setup(props) {
        const { actions, contextMenu, onMenuClick, showContextMenu } = useContextMenu(props.group, "horizontal");
        const { dragOver, dragStart, drop } = useDragDrop(props.group);
        const navClass = vue.computed(() => ({
            "vp-pills": "pills" === props.appearance,
            "vp-tabs": "tabs" === props.appearance,
            "vp-tabs-bottom": "top" === props.position,
        }));
        const scrollbarClass = vue.computed(() => ({
            "before:vp-bottom-px": "tabs" === props.appearance && "bottom" === props.position,
            "before:vp-content-[''] before:vp-block before:vp-absolute before:vp-left-0 before:vp-w-full before:vp-h-px before:vp-bg-gray-300": "tabs" === props.appearance,
            "before:vp-top-px": "tabs" === props.appearance && "top" === props.position,
        }));
        const { tabs } = vue.toRefs(props);
        const tabRefs = [];
        vue.watch(tabs, (newVal) => __awaiter(this, void 0, void 0, function* () {
            yield vue.nextTick();
            tabRefs.length = newVal.length;
            for (let i = 0, l = tabRefs.length; i < l; i++) {
                let left = 0;
                const el = tabRefs[i].$el;
                for (let j = i; 0 <= --j;) {
                    const prevEl = tabRefs[j].$el;
                    left += prevEl.getBoundingClientRect().width;
                }
                el.style.setProperty("left", `${left}px`);
            }
        }), {
            deep: true,
            immediate: true,
        });
        function createTabRef(el, index) {
            if (el)
                tabRefs[index] = el;
        }
        return {
            actions,
            contextMenu,
            createTabRef,
            dragOver,
            dragStart,
            drop,
            navClass,
            onMenuClick,
            scrollbarClass,
            showContextMenu,
            tabRefs,
        };
    },
});

const _hoisted_1$3 = { key: 0 };

function render$3(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_tab = vue.resolveComponent("tab");
  const _component_context_menu_list = vue.resolveComponent("context-menu-list");
  const _component_context_menu = vue.resolveComponent("context-menu");
  const _component_x_scrollbar = vue.resolveComponent("x-scrollbar");

  return (vue.openBlock(), vue.createBlock(_component_x_scrollbar, {
    class: vue.normalizeClass(_ctx.scrollbarClass)
  }, {
    default: vue.withCtx(() => [
      vue.createElementVNode("nav", {
        class: vue.normalizeClass(["vp-flex vp-flex-nowrap vp-border-0 vp-m-0 vp-p-0 vp-h-10 vp-box-border vp-bg-transparent vp-select-none vp-relative vp-nav", _ctx.navClass]),
        onDragstart: _cache[0] || (_cache[0] = (...args) => (_ctx.dragStart && _ctx.dragStart(...args))),
        onDragover: _cache[1] || (_cache[1] = (...args) => (_ctx.dragOver && _ctx.dragOver(...args))),
        onDrop: _cache[2] || (_cache[2] = (...args) => (_ctx.drop && _ctx.drop(...args)))
      }, [
        (vue.openBlock(true), vue.createElementBlock(vue.Fragment, null, vue.renderList(_ctx.tabs, (tab, i) => {
          return (vue.openBlock(), vue.createBlock(_component_tab, {
            ref_for: true,
            ref: el => _ctx.createTabRef(el, i),
            key: tab.id,
            "tab-id": tab.id,
            active: tab.active,
            closable: tab.closable,
            title: tab.hoverTitle,
            group: _ctx.group,
            rounded: _ctx.rounded,
            direction: _ctx.position,
            class: "vp-top-0",
            onContextmenu: vue.withModifiers($event => (_ctx.showContextMenu($event, tab.id)), ["prevent"])
          }, {
            default: vue.withCtx(() => [
              (typeof tab.title === 'string')
                ? (vue.openBlock(), vue.createElementBlock("span", _hoisted_1$3, vue.toDisplayString(tab.title), 1))
                : (vue.openBlock(), vue.createBlock(vue.resolveDynamicComponent(tab.title), vue.normalizeProps(vue.mergeProps({ key: 1 }, tab.titleProps)), null, 16))
            ]),
            _: 2
          }, 1032, ["tab-id", "active", "closable", "title", "group", "rounded", "direction", "onContextmenu"]))
        }), 128)),
        vue.createVNode(_component_context_menu, {
          ref: "contextMenu",
          boundingView: ".vp-nav"
        }, {
          default: vue.withCtx(() => [
            vue.createVNode(_component_context_menu_list, {
              actions: _ctx.actions,
              onMenuClick: _ctx.onMenuClick
            }, null, 8, ["actions", "onMenuClick"])
          ]),
          _: 1
        }, 512)
      ], 34)
    ]),
    _: 1
  }, 8, ["class"]))
}

script$3.render = render$3;

var script$2 = vue.defineComponent({
    name: "TabView",
    props: {
        group: {
            required: true,
            type: String,
        },
    },
    setup(props) {
        const currentTab = useCurrentTab(props.group);
        const iframes = vue.ref([]);
        vue.watch(currentTab, (newVal) => {
            if (newVal.hasIframe && !iframes.value.some((tab) => tab._key === newVal._key)) {
                iframes.value = iframes.value.filter((tab) => { var _a; return !((_a = tab._key) === null || _a === void 0 ? void 0 : _a.startsWith(newVal.id + "~")); });
                iframes.value.push(newVal);
            }
        }, { immediate: true });
        return {
            currentTab,
            iframes,
        };
    },
});

const _hoisted_1$2 = {
  role: "tabpanel",
  class: "vp-box-border"
};

function render$2(_ctx, _cache, $props, $setup, $data, $options) {
  return (vue.openBlock(), vue.createElementBlock("div", _hoisted_1$2, [
    (vue.openBlock(), vue.createBlock(vue.KeepAlive, null, [
      (!_ctx.currentTab.hasIframe)
        ? (vue.openBlock(), vue.createBlock(vue.resolveDynamicComponent(_ctx.currentTab.view), vue.mergeProps({
            key: 0,
            key: _ctx.currentTab._key
          }, _ctx.currentTab.viewProps), null, 16))
        : vue.createCommentVNode("", true)
    ], 1024)),
    (vue.openBlock(true), vue.createElementBlock(vue.Fragment, null, vue.renderList(_ctx.iframes, (tab) => {
      return vue.withDirectives((vue.openBlock(), vue.createElementBlock("div", {
        key: tab._key
      }, [
        (vue.openBlock(), vue.createBlock(vue.resolveDynamicComponent(tab.view), vue.normalizeProps(vue.guardReactiveProps(tab.viewProps)), null, 16))
      ])), [
        [vue.vShow, _ctx.currentTab._key === tab._key]
      ])
    }), 128))
  ]))
}

script$2.render = render$2;

var script$1 = vue.defineComponent({
    name: "YScrollbar",
    setup() {
        const navWrapper = vue.ref(null);
        const track = vue.ref(null);
        const thumb = vue.ref(null);
        const scrollRatio = vue.ref(1);
        const showScrollbar = vue.ref(false);
        const thumbStyle = vue.computed(() => (!showScrollbar.value || 1 === scrollRatio.value ? null : { height: `${scrollRatio.value * 100}%` }));
        const navWrapperScrollPos = vue.reactive({
            top: 0,
            y: 0,
        });
        function startGrabThumb(e) {
            navWrapperScrollPos.top = navWrapper.value ? navWrapper.value.scrollTop : 0;
            navWrapperScrollPos.y = e.clientY;
            document.body.style.setProperty("user-select", "none");
            document.addEventListener("mousemove", moveThumb, false);
            document.addEventListener("mouseup", stopGrabThumb, false);
        }
        function moveThumb(e) {
            if (navWrapper.value) {
                navWrapper.value.scrollTop = (e.clientY - navWrapperScrollPos.y) / scrollRatio.value + navWrapperScrollPos.top;
            }
        }
        function stopGrabThumb() {
            document.body.style.removeProperty("user-select");
            document.removeEventListener("mousemove", moveThumb, false);
            document.removeEventListener("mouseup", stopGrabThumb, false);
        }
        function scrollThumb() {
            window.requestAnimationFrame(() => {
                if (navWrapper.value && thumb.value) {
                    thumb.value.style.setProperty("top", `${navWrapper.value.scrollTop / navWrapper.value.scrollHeight * 100}%`);
                }
            });
        }
        function onClickTrack(e) {
            if (!track.value || !navWrapper.value)
                return;
            const trackRect = track.value.getBoundingClientRect();
            const percentage = (e.clientY - trackRect.top) / trackRect.height;
            navWrapper.value.scrollTop = percentage * (navWrapper.value.scrollHeight - navWrapper.value.clientHeight);
        }
        function trackUserMovement(e) {
            var _a;
            if (!navWrapper.value)
                return;
            scrollRatio.value = navWrapper.value.clientHeight / navWrapper.value.scrollHeight;
            showScrollbar.value = "mouseenter" === e.type && 1 > scrollRatio.value;
            (_a = thumb.value) === null || _a === void 0 ? void 0 : _a.style.setProperty("top", `${navWrapper.value.scrollTop / navWrapper.value.scrollHeight * 100}%`);
        }
        return {
            navWrapper,
            onClickTrack,
            scrollThumb,
            showScrollbar,
            startGrabThumb,
            thumb,
            thumbStyle,
            track,
            trackUserMovement,
        };
    },
});

const _hoisted_1$1 = { class: "vp-absolute vp-h-full vp-top-0 vp-right-0 vp-w-1.5" };

function render$1(_ctx, _cache, $props, $setup, $data, $options) {
  return (vue.openBlock(), vue.createElementBlock("div", {
    class: "vp-overflow-hidden vp-h-10 vp-relative",
    onMouseenter: _cache[3] || (_cache[3] = (...args) => (_ctx.trackUserMovement && _ctx.trackUserMovement(...args))),
    onMouseleave: _cache[4] || (_cache[4] = (...args) => (_ctx.trackUserMovement && _ctx.trackUserMovement(...args)))
  }, [
    vue.createElementVNode("div", {
      ref: "navWrapper",
      class: "vp-h-full vp-overflow-y-auto vp-overflow-x-hidden -vp-mr-4 vp-pr-4 vp-box-border",
      onScroll: _cache[0] || (_cache[0] = (...args) => (_ctx.scrollThumb && _ctx.scrollThumb(...args)))
    }, [
      vue.renderSlot(_ctx.$slots, "default")
    ], 544),
    vue.withDirectives(vue.createElementVNode("div", _hoisted_1$1, [
      vue.createElementVNode("div", {
        ref: "track",
        class: "vp-absolute vp-top-0 vp-left-0 vp-w-full vp-h-full vp-bg-slate-400/30 vp-rounded",
        onClick: _cache[1] || (_cache[1] = (...args) => (_ctx.onClickTrack && _ctx.onClickTrack(...args)))
      }, null, 512),
      vue.createElementVNode("div", {
        ref: "thumb",
        class: "vp-absolute vp-left-0 vp-w-full vp-bg-zinc-400 vp-rounded",
        style: vue.normalizeStyle(_ctx.thumbStyle),
        onMousedown: _cache[2] || (_cache[2] = (...args) => (_ctx.startGrabThumb && _ctx.startGrabThumb(...args)))
      }, null, 36)
    ], 512), [
      [vue.vShow, _ctx.showScrollbar]
    ])
  ], 32))
}

script$1.render = render$1;

var script = vue.defineComponent({
    name: "VTabs",
    components: {
        ContextMenu: script$7,
        ContextMenuList: script$6,
        Tab: script$5,
        YScrollbar: script$1,
    },
    props: {
        group: {
            required: true,
            type: String,
        },
        rounded: {
            default: false,
            type: Boolean,
        },
        tabs: {
            required: true,
            type: Array,
        },
    },
    setup(props) {
        const { actions, contextMenu, onMenuClick, showContextMenu } = useContextMenu(props.group, "vertical");
        const { dragOver, dragStart, drop } = useDragDrop(props.group);
        return {
            actions,
            contextMenu,
            dragOver,
            dragStart,
            drop,
            onMenuClick,
            showContextMenu,
        };
    },
});

const _hoisted_1 = { key: 0 };

function render(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_tab = vue.resolveComponent("tab");
  const _component_context_menu_list = vue.resolveComponent("context-menu-list");
  const _component_context_menu = vue.resolveComponent("context-menu");
  const _component_y_scrollbar = vue.resolveComponent("y-scrollbar");

  return (vue.openBlock(), vue.createBlock(_component_y_scrollbar, null, {
    default: vue.withCtx(() => [
      vue.createElementVNode("nav", {
        class: "vp-flex vp-flex-col vp-border-0 vp-m-0 vp-p-0 vp-box-border vp-bg-transparent vp-pills vp-select-none vp-relative vp-nav",
        onDragstart: _cache[0] || (_cache[0] = (...args) => (_ctx.dragStart && _ctx.dragStart(...args))),
        onDragover: _cache[1] || (_cache[1] = (...args) => (_ctx.dragOver && _ctx.dragOver(...args))),
        onDrop: _cache[2] || (_cache[2] = (...args) => (_ctx.drop && _ctx.drop(...args)))
      }, [
        (vue.openBlock(true), vue.createElementBlock(vue.Fragment, null, vue.renderList(_ctx.tabs, (tab, i) => {
          return (vue.openBlock(), vue.createBlock(_component_tab, {
            key: tab.id,
            "tab-id": tab.id,
            active: tab.active,
            closable: tab.closable,
            title: tab.hoverTitle,
            group: _ctx.group,
            rounded: _ctx.rounded,
            style: vue.normalizeStyle({ top: `${2.5 * i}rem` }),
            direction: "none",
            class: "vp-left-0 vp-w-full",
            onContextmenu: vue.withModifiers($event => (_ctx.showContextMenu($event, tab.id)), ["prevent"])
          }, {
            default: vue.withCtx(() => [
              (typeof tab.title === 'string')
                ? (vue.openBlock(), vue.createElementBlock("span", _hoisted_1, vue.toDisplayString(tab.title), 1))
                : (vue.openBlock(), vue.createBlock(vue.resolveDynamicComponent(tab.title), vue.normalizeProps(vue.mergeProps({ key: 1 }, tab.titleProps)), null, 16))
            ]),
            _: 2
          }, 1032, ["tab-id", "active", "closable", "title", "group", "rounded", "style", "onContextmenu"]))
        }), 128)),
        vue.createVNode(_component_context_menu, {
          ref: "contextMenu",
          boundingView: ".vp-nav"
        }, {
          default: vue.withCtx(() => [
            vue.createVNode(_component_context_menu_list, {
              actions: _ctx.actions,
              onMenuClick: _ctx.onMenuClick
            }, null, 8, ["actions", "onMenuClick"])
          ]),
          _: 1
        }, 512)
      ], 32)
    ]),
    _: 1
  }))
}

script.render = render;

var index = {
    install(app, options = {}) {
        app.component(script$3.name, script$3);
        app.component(script.name, script);
        app.component(script$2.name, script$2);
        app.provide(key, mergeWithDefaultLocales(options));
    },
};

exports.addTab = addTab;
exports.createTabs = createTabs;
exports["default"] = index;
exports.onTabActivated = onTabActivated;
exports.onTabDeactivated = onTabDeactivated;
exports.refreshTab = refreshTab;
exports.removeAbove = removeAbove;
exports.removeAll = removeAll;
exports.removeBelow = removeBelow;
exports.removeLeft = removeLeft;
exports.removeOthers = removeOthers;
exports.removeRight = removeRight;
exports.removeTab = removeTab;
exports.useCurrentTab = useCurrentTab;
exports.useTabs = useTabs;
